#ifndef TUCSENCAMRET_ERROR_MAP_H
#define TUCSENCAMRET_ERROR_MAP_H

#include <unordered_map>
#include <string>

// Tabella degli errori come mappa statica
static const std::unordered_map<uint32_t, std::string> TUCSEN_ERR_MSG_map = {
    {0x00000001, "no error, general success code, app should check the value is positive"},
    {0x80000000, "error"},
    {0x80000101, "not enough memory"},
    {0x80000102, "not enough resource except memory"},
    {0x80000103, "no sub module"},
    {0x80000104, "no driver"},
    {0x80000105, "no camera"},
    {0x80000106, "no grabber"},
    {0x80000107, "there is no alternative or influence id, or no more property id"},
    {0x80000110, "fail open the camera"},
    {0x80000111, "fail open the bulk in endpoint"},
    {0x80000112, "fail open the bulk out endpoint"},
    {0x80000113, "fail open the control endpoint"},
    {0x80000114, "fail close the camera"},
    {0x80000115, "fail open the file"},
    {0x80000116, "fail open the video codec"},
    {0x80000117, "fail open the video context"},
    {0x80000201, "API requires has not initialized state."},
    {0x80000202, "API cannot process in busy state."},
    {0x80000203, "API requires has initialized state."},
    {0x80000204, "some resource is exclusive and already used."},
    {0x80000205, "API requires busy state."},
    {0x80000206, "API requires ready state."},
    {0x80000207, "abort process"},
    {0x80000208, "timeout"},
    {0x80000209, "frame data is lost"},
    {0x8000020A, "frame is lost but reason is low lever driver's bug"},
    {0x8000020B, "the USB status error"},
    {0x80000301, "invalid camera"},
    {0x80000302, "invalid camera handle"},
    {0x80000303, "invalid the option value of structure"},
    {0x80000304, "invalid property id"},
    {0x80000305, "invalid capability id"},
    {0x80000306, "invalid parameter id"},
    {0x80000307, "invalid parameter"},
    {0x80000308, "invalid frame index"},
    {0x80000309, "invalid property value"},
    {0x8000030A, "invalid property value equal"},
    {0x8000030B, "the property id specifies channel but channel is invalid"},
    {0x8000030C, "the combination of subarray values are invalid"},
    {0x8000030D, "invalid view window handle"},
    {0x8000030E, "invalid file path"},
    {0x8000030F, "invalid vendor property id"},
    {0x80000310, "the property does not have value text"},
    {0x80000311, "value is out of range"},
    {0x80000312, "camera does not support the function or property with current settings"},
    {0x80000313, "the property is not writable"},
    {0x80000314, "the property is not readable"},
    {0x80000410, "unexpected error from camera"},
    {0x80000411, "new API required for this value"},
    {0x80000412, "property cannot access during this status"},
    {0x80000501, "no dark and shading correction data"},
    {0x80000601, "invalid profiles set name"},
    {0x80000602, "invalid process property id"},
    {0x80000701, "image decoding failure"},
    {0x80000702, "image data copying failure"},
    {0x80000703, "image encoding to video failure"},
    {0x80000704, "write video frame failure"},
    {0x83001001, "fail read from camera"},
    {0x83001002, "fail write to camera"},
    {0x83001003, "optics part unplugged"},
    {0x00000002, "no error, vendor receive frame message"},
    {0x00000003, "no error, receive the external trigger signal"}
};

static const std::unordered_map<uint32_t, std::string> TUCSEN_TRIG_MODE_map = {
    {0x00, "capture start sequence mode (free running)"},
    {0x01, "capture start trigger standard mode"},
    {0x02, "capture start trigger synchronous mode"},
    {0x03, "capture start trigger global"},
    {0x04, "capture start trigger software"}
};

static const std::unordered_map<uint32_t, std::string> TUCSEN_TRIGGER_EXP_map = {
    {0x00, "use exposure time"},
    {0x01, "use width level"}
};

static const std::unordered_map<uint32_t, std::string> TUCSEN_TRIGGER_EDGE_map = {
    {0x01, "rising edge"},
    {0x00, "failing edge"}
};

static const std::unordered_map<uint32_t, std::string> TUCSEN_OUTPUTTRG_PORT_map = {
    {0x00, "use port1"},
    {0x01, "use port2"},
    {0x02, "use port3"}
};

static const std::unordered_map<uint32_t, std::string> TUCSEN_OUTPUTTRG_KIND_map = {
    {0x00, "use low gnd"},
    {0x01, "use high vcc"},
    {0x02, "use trigger input"},
    {0x03, "use exposure start"},
    {0x04, "use global exposure"},
    {0x05, "use read end"}
};

static const std::unordered_map<uint32_t, std::string> TUCSEN_OUTPUTTRG_EDGE_map = {
    {0x00, "rising edge"},
    {0x01, "failing edge"}
};

// Return a string depending on the hex input value
// inline to be copied to every cpp it includes itself
inline std::string GetTUCSENString(const std::unordered_map<uint32_t, std::string>& table, uint32_t hexValue) {
    auto it = table.find(hexValue);
    if (it != table.end()) {
        return it->second;
    } else {
        return "Unknown hex value";
    }
}

#endif // TUCSENCAMRET_ERROR_MAP_H

